import {Backend, DataSource} from '../backend';
import {Operation} from './operation';
import {App} from './app';
import {Endpoint} from './endpoint';

export interface RecipeState<A extends App = App> {
	app: A;
	backend?: Backend<A>;
	dataSource?: DataSource;
	operations?: Record<string, Operation>;
	endpoints?: Record<string, Endpoint>;
}

export type Recipe<A extends App = App, B extends A = A> = (a: RecipeState<A>) => RecipeState<B>;

export const composeRecipes = (recipes: Recipe[]): Recipe => (params) => (
	recipes.reduce(
		(rr, r) => r(rr),
		params
	)
);
