import {Recipe} from '../../common/recipe';
import {endpoint, operation, validation as v} from '../../common';
import {backend, DataSource} from '../../backend';
import {
	DataSourceNotFoundResponseError,
	ItemNotFoundReponseError,
	ResourceCollectionFetchedResponse,
	ResourceItemFetchedResponse,
} from './response';

interface AddResourceRecipeParams {
	endpointName: string;
	dataSource?: DataSource;
}

export const addResourceRecipe = (params: AddResourceRecipeParams): Recipe => (a) => {
	const operations = {
		fetch: operation({
			name: 'fetch' as const,
		}),
	};

	const theEndpoint = endpoint({
		name: params.endpointName,
		schema: v.object({
			username: v.string(),
		}),
	})
		.param('resourceId')
		.can('fetch');

	const enhancedApp = a.app
		.operation(operations.fetch)
		.endpoint(theEndpoint);

	const theBackend = a.backend ?? backend({
		app: enhancedApp,
		dataSource: params.dataSource,
	});

	theBackend
		.implementOperation('fetch', async (ctx) => {
			const dataSource: DataSource = ctx.endpoint.dataSource ?? theBackend.dataSource ?? {} as DataSource;
			// need to genericise the response here so we don't depend on the HTTP responses.
			const { resourceId } = ctx.params;
			const { getById, getMultiple } = dataSource;

			if (typeof resourceId === 'undefined') {
				if (typeof getMultiple === 'undefined') {
					throw new DataSourceNotFoundResponseError();
				}

				// TODO add query here
				const items = await getMultiple();
				return new ResourceCollectionFetchedResponse({
					statusMessage: 'Resource Collection Fetched',
					body: items,
				});
			}

			if (typeof getById === 'undefined') {
				throw new DataSourceNotFoundResponseError();
			}

			const item = await getById(resourceId);
			if (!item) {
				throw new ItemNotFoundReponseError();
			}

			return new ResourceItemFetchedResponse({
				statusMessage: 'Resource Item Fetched',
				body: item,
			});
		});

	return {
		operations,
		app: enhancedApp,
		backend: theBackend,
		endpoints: {
			[params.endpointName]: theEndpoint,
		},
	};
};
